//////////////////////////////////////////////////////////////////////////////
//
//                  ELEKTOR RFID READER for MIFARE and ISO14443-A
//
//                      Copyright (c) 2006 Gerhard H. Schalk
//         
//////////////////////////////////////////////////////////////////////////////
#include "RC522_Mifare.h"
#include "Mifare_Util.h"
#include "UART.h"
#include "PC_Reader_COM.h"
#include "Mifare_PC_Reader.h" 
#include "LCD.h"
#include "Timer1.h"
#include "EEPROM.h"
#include "Mifare_Access_Control.h"

//------------------------------------------------------------------------
// Function: MIFARE_PC_Reader
//------------------------------------------------------------------------
// Description:
// The MIFARE_PC_Reader is called by main when a new PC command is
// received via Receive_PC_Command function. The Receive_PC_Command
// function is called by the UART ISR.
//
// The MIFARE_PC_Reader function is a simple command dispatcher for 
// all implemented card commands and Reader commands.
//
// PC Command: (PC to Reader)
// gbaSerialRecBuffer  0	 1	   2	  3     ..      n 
//                  --------------------------------------------
//                  | SOH | Cmd | Len | Data[0] .. Data[Len-1] |
//                  --------------------------------------------
//------------------------------------------------------------------------
// Command Response:  (Reader to PC)
//         
//  abResponseBuffer   0	   1	  2	      3     ..      n 
//                  -----------------------------------------------
//                  | SOH | Status | Len | Data[0] .. Data[Len-1] |
//                  -----------------------------------------------
//------------------------------------------------------------------------
void MIFARE_PC_Reader(void)
{
    short status;
    unsigned char idata abResponseBuffer[32], abSNR[8],  bSAK;
    unsigned int addr;
    printf_lcd(1,1,"  PC Mode       ");
    printf_lcd(2,1,"  RcvCmd:0x%b02X     ",gbaSerialRecBuffer[1]);
    switch(gbaSerialRecBuffer[1])
    {

                        //--------------------------------------------------------------	
                        // ISO14443 Activation & Mifare (Ultra Light) Commands
                        //--------------------------------------------------------------
        case 0x20:      // Reader Command: RF - Reset
                        Rc522RFReset(5);    //RF - Reset    
                        SendCmdResponce(0,abResponseBuffer,0);
                        break;

                        // Reader Command: REQA
        case 0x26:      status = Request(ISO14443_3_REQA, &abResponseBuffer);
                        SendCmdResponce(status,abResponseBuffer,2);
                        break;

                        // Reader Command: Mifare / Mifare Ultra Light READ		
        case 0x30:      status = Read(gbaSerialRecBuffer[3],abResponseBuffer);
                        SendCmdResponce(status,abResponseBuffer,16);
                        break;

                        // Reader Command: HALTA
        case 0x50:      status = HaltA();
                        SendCmdResponce(status,abResponseBuffer,0);
                        break;

                        // Reader Command: WUPA
        case 0x52:      status = Request(ISO14443_3_WUPA, &abResponseBuffer);
                        SendCmdResponce(status,abResponseBuffer,2);
                        break;

                        // Reader Command: Activate Card: REQA-AC-SELECT-(AC-SELECT)
        case 0x40:      status = ActivateCard(ISO14443_3_REQA, 
                                              &abResponseBuffer[0],  // 2 Bytes ATQA
                                              &abResponseBuffer[2],  // 4 or 8 Byte UID
                                              &abResponseBuffer[10],  // 1 Byte UID Length
                                              &abResponseBuffer[11]); // 1 Byte SAK
                        SendCmdResponce(status,abResponseBuffer,12);
                        break;

                        // Reader Command: Activate Card: WUPA-AC-SELECT-(AC-SELECT)
        case 0x41:      status = ActivateCard(ISO14443_3_WUPA, 
                                              &abResponseBuffer[0],  // 2 Bytes ATQA
                                              &abResponseBuffer[2],  // 4 or 8 Byte UID
                                              &abResponseBuffer[10],  // 1 Byte UID Length
                                              &abResponseBuffer[11]); // 1 Byte SAK
                        SendCmdResponce(status,abResponseBuffer,12);
                        break;

                        // Reader Command: Activate Card: RF-Reset-REQA-AC-SELECT-(AC-SELECT)
        case 0x42:      Rc522RFReset(5);  //RF - Reset 
                        status = ActivateCard(ISO14443_3_REQA, 
                                              &abResponseBuffer[0],   // 2 Bytes ATQA
                                              &abResponseBuffer[2],   // 4 or 8 Byte UID
                                              &abResponseBuffer[10],  // 1 Byte UID Length
                                              &abResponseBuffer[11]); // 1 Byte SAK
                        SendCmdResponce(status,abResponseBuffer,12);
                        break;

        case 0x60:      // Reader Command: Mifare Authentication with Key A
                        // This command is used for Mifare 1/4K Standard Cards
                        status = Authentication(MIFARE_AUTHENT_A,        // Auth. with MF Key A
                                                &gbaSerialRecBuffer[3],  // 6 Byte Mifare Key A
                                                &gbaSerialRecBuffer[9],  // 4 Byte Card UID
                                                gbaSerialRecBuffer[13]); // 1 Mifare Block
                        SendCmdResponce(status,abResponseBuffer,0);
                        break;  
                        
        case 0x61:      // Reader Command: Mifare Authentication with Key B
                        // This command is used for Mifare 1/4K Standard Cards
                        status = Authentication(MIFARE_AUTHENT_B,        // Auth. with MF Key B
                                                &gbaSerialRecBuffer[3],  // 6 Byte Mifare Key B
                                                &gbaSerialRecBuffer[9],  // 4 Byte Card UID
                                                gbaSerialRecBuffer[13]); // 1 Mifare Block
                        SendCmdResponce(status,abResponseBuffer,0);
                        break;

                         // Reader Command: Anticollision of Cascade Level 1
        case 0x93:      status = CascAnticoll(0x93,0,&abSNR[0]);
                        SendCmdResponce(status,abSNR,4);
                        break;

                         // Reader Command: Select of Cascade Level 1
        case 0x94:      status = Select(0x93, &gbaSerialRecBuffer[3], &bSAK);
                        SendCmdResponce(status,&bSAK,1);
                        break;

                         // Reader Command: Anticollision of Cascade Level 2	
        case 0x95:      status = CascAnticoll(0x95,0,&abSNR[4]);
                        SendCmdResponce(status,&abSNR[4],4);
                        break;

                         // Reader Command: Select of Cascade Level 2
        case 0x96:      status = Select(0x95, &gbaSerialRecBuffer[3], &bSAK);
                        SendCmdResponce(status,&bSAK,1);
                        break; 

        case 0xA0:      // Mifare Write & Mifare Ultra Light Compatibity Write
                        status = Write(gbaSerialRecBuffer[3],&gbaSerialRecBuffer[4]);
                        SendCmdResponce(status,abResponseBuffer,0);
                        break;

        case 0xC0:      // Reader Command: DECREMENT a Mifare Value Block
                        // This command is used for Mifare 1/4K Standard Cards only
                        status = ValueOper(MIFARE_DECREMENT,       // Decrement
                                           gbaSerialRecBuffer[3],  // 1 Byte; Address of the block
                                          &gbaSerialRecBuffer[4],  // 4 Byte Value
                                           gbaSerialRecBuffer[8]); // 1 Byte; trans_addr the address 
                                                              // to save the resulet of inc. or dec.
                        SendCmdResponce(status,abResponseBuffer,0);
                        break; 
                        
        case 0xC1:     // Reader Command: INCREMENT a Mifare Value Block
                       // This command is used for Mifare 1/4K Standard Cards only
                       status = ValueOper(MIFARE_INCREMENT,            // INCREMENT
                                          gbaSerialRecBuffer[3], // 1 Byte; Address of the block
                                          &gbaSerialRecBuffer[4], // 4 Byte Value
                                          gbaSerialRecBuffer[8]); // 1 Byte; trans_addr the address 
                                                              // to save the resulet of inc. or dec.
                        SendCmdResponce(status,abResponseBuffer,0);
                        break;
    
        case 0xC5:  // Reader Command: InitBlock; Initialize a Mifare Value Block
                    // This command is used for Mifare 1/4K Standard Cards only
                    status = InitBlock(gbaSerialRecBuffer[3],  // 1 Byte; Address of the block
                                      &gbaSerialRecBuffer[4]); // 4 Byte Value
                    SendCmdResponce(status,abResponseBuffer,0);
                    break;                                                  


                        //--------------------------------------------------------------	
                        // Reader Internal Commands
                        //--------------------------------------------------------------

                        // Set Reader Main State Machine = Terminal Mode
        case 0xF0:      gbMainState = MainState_TerminalMode;
                        SendCmdResponce(0,abResponseBuffer,0);
                        Rc522Init();
                        break;

                        // Set Reader Main State Machine = Reader Mode
        case 0xF1:      gbMainState = MainState_PCReaderMode;
                        SendCmdResponce(0,abResponseBuffer,0);
                        Rc522Init();
                        printf_lcd(1,1,"  PC Mode       ");
                        printf_lcd(2,1,"  PC:115200 Bd  ");
                        break;

        case 0xF2:      // Set PC-Reader RS232/USB Baudrate
                        // The communication Baudrate has no major 
                        // influence  of the overall performance.
                        // Therefore there is no need to change the 
                        // Baudrate.
                        SendCmdResponce(0,abResponseBuffer,0); 
                        switch(gbaSerialRecBuffer[3])
                        {
                            case 0: 
                                uart_init(9600);
                                printf_lcd(2,1,"  PC: 9600 Bd   ");
                                break;
                            case 1: 
                                uart_init(19200);
                                printf_lcd(2,1,"  PC: 19200 Bd  ");
                                break;
                            case 2: 
                                uart_init(38400);
                                printf_lcd(2,1,"  PC: 38400 Bd  ");
                                break;
                            case 3: 
                                uart_init(57600);
                                printf_lcd(2,1,"  PC: 57600 Bd  ");
                                break;
                            case 4: 
                                uart_init(115200);
                                printf_lcd(2,1,"  PC:115200 Bd  ");
                                break;
                            case 5: 
                                uart_init(230400);
                                printf_lcd(2,1,"  PC:230400 Bd  ");
                                break;
                            case 6: 
                                uart_init(460800);
                                printf_lcd(2,1,"  PC:460800 Bd  ");
                                break;
                            default:
                                uart_init(115200);
                                printf_lcd(2,1,"  PC:115200 Bd  ");
                        }
                        break;

                        // Set Reader Main State Machine = MainState_ACCESS_CONTROL
        case 0xF3:      gbMainState = MainState_ACCESS_CONTROL;
                        SendCmdResponce(0,abResponseBuffer,0);
                        Read_EE_AccessControl_LCD_Content();
                        Rc522Init();
                        break;

                        
         case 0xF5:     // LPC936 EEPROM WRITE (First 128 EEPROM Bytes)
                        addr = gbaSerialRecBuffer[3] * 256 + gbaSerialRecBuffer[4];
                        WriteEEPROM(addr,gbaSerialRecBuffer[5]);
                        SendCmdResponce(0,abResponseBuffer,0);
                        
                        break;


         case 0xF6:     // LPC936 EEPROM Read (First 128 EEPROM Bytes)
                        addr = gbaSerialRecBuffer[3] * 256 + gbaSerialRecBuffer[4];
                        abResponseBuffer[0] = ReadEEPROM(addr);
                        SendCmdResponce(0,abResponseBuffer,1);
                        break;

                        // LPC936 Soft Reset
        case 0xFF:      SendCmdResponce(0,abResponseBuffer,0);
                        Timer1_Delay_ms(5);
                        ((void (code *)(void)) 0) ();
                        break;
    }
}